import pandas as pd
import numpy as np

# columns for vehicle motion DataFrame
column_list = ['b_lfc', 'b_rfc', 'b_rrc', 'b_lrc', 'lfw', 'lfw_a', 'lfw_b',
               'lfw_c', 'lfw_d', 'rfw', 'rfw_a', 'rfw_b', 'rfw_c', 'rfw_d',
                'rrw', 'rrw_a',	'rrw_b', 'rrw_c', 'rrw_d', 'lrw',  'lrw_a',
                'lrw_b', 'lrw_c', 'lrw_d', 'cg', 'xaxis', 'yaxis', 'vel_v']

def position_data_motion(veh):
    """
    set striking to True to return location of impact point in vehicle 1
    """
    # initialize empty dataframe
    veh.p_vx = pd.DataFrame(columns = column_list)                                      # time variant vehicle geometry for plotting in vehicle frame
    veh.p_vy = pd.DataFrame(columns = column_list)                                      # time variant vehicle geometry for plotting in vehicle frame

    # loop for creating seperate dataframes for X and Y components of vehicle drawing in vehicle frame
    for i in (range(len(veh.model.t))):
        """
        vehicle frame coordinates for body geometry, tire positions and velocity vector
        if MV = True - impact point and edge points will be tansformed here as well
        """

        veh.p_vx = veh.p_vx.append({'b_lfc': veh.lcgf + veh.f_hang,     # body outline
                                  'b_rfc': veh.lcgf + veh.f_hang,
                                  'b_rrc': -1 * (veh.lcgr + veh.r_hang),
                                  'b_lrc': -1 * (veh.lcgr + veh.r_hang),
                                  'lfw':   veh.lcgf,                         # left front wheel
                                  'lfw_a': veh.lcgf + veh.tire_d / 2 * np.cos(veh.model.loc[i,'delta_rad']) - -1 * veh.tire_w / 2 * np.sin(veh.model.loc[i,'delta_rad']),
                                  'lfw_b': veh.lcgf + veh.tire_d / 2 * np.cos(veh.model.loc[i,'delta_rad']) - veh.tire_w / 2 * np.sin(veh.model.loc[i,'delta_rad']),
                                  'lfw_c': veh.lcgf + -1 * veh.tire_d / 2 * np.cos(veh.model.loc[i,'delta_rad']) - veh.tire_w / 2 * np.sin(veh.model.loc[i,'delta_rad']),
                                  'lfw_d': veh.lcgf + -1 * veh.tire_d / 2 * np.cos(veh.model.loc[i,'delta_rad']) - -1 * veh.tire_w / 2*np.sin(veh.model.loc[i,'delta_rad']),
                                  'rfw':   veh.lcgf,                         # Right front wheel
                                  'rfw_a': veh.lcgf + veh.tire_d / 2 * np.cos(veh.model.loc[i,'delta_rad']) - -1 * veh.tire_w / 2 * np.sin(veh.model.loc[i,'delta_rad']),
                                  'rfw_b': veh.lcgf + veh.tire_d / 2 * np.cos(veh.model.loc[i,'delta_rad']) - veh.tire_w / 2 * np.sin(veh.model.loc[i,'delta_rad']),
                                  'rfw_c': veh.lcgf + -1 * veh.tire_d / 2 * np.cos(veh.model.loc[i,'delta_rad']) - veh.tire_w / 2 * np.sin(veh.model.loc[i,'delta_rad']),
                                  'rfw_d': veh.lcgf + -1 * veh.tire_d / 2*np.cos(veh.model.loc[i,'delta_rad']) - -1 * veh.tire_w / 2 * np.sin(veh.model.loc[i,'delta_rad']),
                                  'rrw':   -1 * veh.lcgr,                          # Right rear wheel
                                  'rrw_a': -1 * veh.lcgr + veh.tire_d / 2,
                                  'rrw_b': -1 * veh.lcgr + veh.tire_d / 2,
                                  'rrw_c': -1 * veh.lcgr - veh.tire_d / 2,
                                  'rrw_d': -1 * veh.lcgr - veh.tire_d / 2,
                                  'lrw':   -1 * veh.lcgr,                          # Left rear wheel
                                  'lrw_a': -1 * veh.lcgr + veh.tire_d / 2,
                                  'lrw_b': -1 * veh.lcgr + veh.tire_d / 2,
                                  'lrw_c': -1 * veh.lcgr - veh.tire_d / 2,
                                  'lrw_d': -1 * veh.lcgr - veh.tire_d / 2,
                                  'cg': 0,                                         # CG
                                  'xaxis': veh.lcgf + veh.f_hang + 1.5,            # line for x-axis
                                  'yaxis': 0,                                      # line for y-axis
                                  'vel_v': veh.model.vx[i]}, ignore_index=True)    # line for velocity vector

        veh.p_vy = veh.p_vy.append({'b_lfc': -1 * veh.width / 2,     # body outline
                                  'b_rfc': veh.width / 2,
                                  'b_rrc': veh.width / 2,
                                  'b_lrc': -1 * veh.width / 2,
                                  'lfw':   -1 * (veh.width / 2 - veh.tire_w / 2),                         # left front wheel
                                  'lfw_a': -1 * (veh.width / 2 - veh.tire_w / 2) + veh.tire_d / 2 * np.sin(veh.model.loc[i,'delta_rad']) + -1 * veh.tire_w / 2 * np.cos(veh.model.loc[i,'delta_rad']),
                                  'lfw_b': -1 * (veh.width / 2 - veh.tire_w / 2) + veh.tire_d / 2 *np.sin(veh.model.loc[i,'delta_rad']) + veh.tire_w / 2 * np.cos(veh.model.loc[i,'delta_rad']),
                                  'lfw_c': -1 * (veh.width / 2 - veh.tire_w / 2) + -1 * veh.tire_d / 2 * np.sin(veh.model.loc[i,'delta_rad']) + veh.tire_w / 2 * np.cos(veh.model.loc[i,'delta_rad']),
                                  'lfw_d': -1 * (veh.width / 2 - veh.tire_w / 2) + -1 * veh.tire_d / 2 * np.sin(veh.model.loc[i,'delta_rad']) + -1 * veh.tire_w / 2 * np.cos(veh.model.loc[i,'delta_rad']),
                                  'rfw':   (veh.width / 2 - veh.tire_w / 2),                         # Right front wheel
                                  'rfw_a': (veh.width / 2 - veh.tire_w / 2) + veh.tire_d / 2 * np.sin(veh.model.loc[i,'delta_rad']) + -1 * veh.tire_w / 2 * np.cos(veh.model.loc[i,'delta_rad']),
                                  'rfw_b': (veh.width / 2 - veh.tire_w / 2) + veh.tire_d / 2 * np.sin(veh.model.loc[i,'delta_rad']) + veh.tire_w / 2 * np.cos(veh.model.loc[i,'delta_rad']),
                                  'rfw_c': (veh.width / 2 - veh.tire_w / 2) + -1*veh.tire_d / 2 * np.sin(veh.model.loc[i,'delta_rad']) + veh.tire_w / 2 * np.cos(veh.model.loc[i,'delta_rad']),
                                  'rfw_d': (veh.width / 2 - veh.tire_w / 2) + -1*veh.tire_d / 2 * np.sin(veh.model.loc[i,'delta_rad']) + -1 * veh.tire_w / 2 * np.cos(veh.model.loc[i,'delta_rad']),
                                  'rrw':   (veh.width / 2 - veh.tire_w / 2),                          # Right rear wheel
                                  'rrw_a': veh.width / 2 - veh.tire_w,
                                  'rrw_b': veh.width / 2,
                                  'rrw_c': veh.width / 2,
                                  'rrw_d': veh.width / 2 - veh.tire_w,
                                  'lrw':   -1 * (veh.width / 2 - veh.tire_w / 2),                          # Left rear wheel
                                  'lrw_a': -1 * veh.width / 2,
                                  'lrw_b': -1 * veh.width / 2 + veh.tire_w,
                                  'lrw_c': -1 * veh.width / 2 + veh.tire_w,
                                  'lrw_d': -1 * veh.width / 2,
                                  'cg':    0,                                         # CG
                                  'xaxis': 0,                                         # line for x-axis
                                  'yaxis': veh.width / 2 + 1.5,                     # line for y-axis
                                  'vel_v': veh.model.vy[i]}, ignore_index=True)       # line for velocity vector

    cgx = veh.model[['Dx']].copy()                          # CG location in inertial frame
    cgy = veh.model[['Dy']].copy()



    if veh.striking:  # impact point (veh1 - striking vehicle)
        # add impact point locations in vehicle frame
        i=0
        for impPoint in veh.impact_points:
            veh.p_vx[f'POI_{i}'] = impPoint[0]
            veh.p_vy[f'POI_{i}'] = impPoint[1]

            norm_length = veh.width
            tang_length = veh.width / 2

            veh.p_vx[f'impact_{i}_norm'] = impPoint[0] + norm_length * np.cos(impPoint[2] * np.pi / 180)
            veh.p_vy[f'impact_{i}_norm'] = impPoint[1] + norm_length * np.sin(impPoint[2] * np.pi / 180)

            veh.p_vx[f'impact_{i}_tang'] = impPoint[0] + tang_length * np.cos(impPoint[2] * np.pi / 180 + np.pi / 2)
            veh.p_vy[f'impact_{i}_tang'] = impPoint[1] + tang_length * np.sin(impPoint[2] * np.pi / 180 + np.pi / 2)

            i+=1

    veh.p_gx = veh.p_vx.copy()                              # time variant vehicle geometry for plotting motion in inertial frame
    veh.p_gy = veh.p_vy.copy()                              # time variant vehicle geometry for plotting motion in inertial frame

    for i in range(0, len(veh.p_gx)):
           for j in range(0, len(veh.p_vx.columns)):
              # coordinate transformation - rotating vehicle frame to draw vehicle
              veh.p_gx.iloc[i, j] = cgx.Dx[i] + veh.p_vx.iloc[i, j] * np.cos(veh.model.loc[i, 'theta_rad']) - veh.p_vy.iloc[i, j] * np.sin(veh.model.loc[i, 'theta_rad'])
              veh.p_gy.iloc[i, j] = cgy.Dy[i] + veh.p_vx.iloc[i, j] * np.sin(veh.model.loc[i, 'theta_rad']) + veh.p_vy.iloc[i, j] * np.cos(veh.model.loc[i, 'theta_rad'])

    # copy locked info from vehicle model
    veh.p_gx['lf_lock'] = veh.model.lf_lock.copy()
    veh.p_gx['rf_lock'] = veh.model.rf_lock.copy()
    veh.p_gx['rr_lock'] = veh.model.rr_lock.copy()
    veh.p_gx['lr_lock'] = veh.model.lr_lock.copy()

    return veh




def position_data_static(vehicle_list):
    """
    static data for plotting vehicle location in global frame
    """

    for veh in vehicle_list:

        veh.px = {'b_lfc': veh.lcgf + veh.f_hang,     # body outline
                  'b_rfc': veh.lcgf + veh.f_hang,
                  'b_rrc': -1 * (veh.lcgr + veh.r_hang),
                  'b_lrc': -1 * (veh.lcgr + veh.r_hang),
                  'lfw':   veh.lcgf,                         # left front wheel
                  'lfw_a': veh.lcgf + veh.tire_d / 2,
                  'lfw_b': veh.lcgf + veh.tire_d / 2,
                  'lfw_c': veh.lcgf + -1 * veh.tire_d / 2,
                  'lfw_d': veh.lcgf + -1 * veh.tire_d / 2,
                  'rfw':   veh.lcgf,                         # Right front wheel
                  'rfw_a': veh.lcgf + veh.tire_d / 2,
                  'rfw_b': veh.lcgf + veh.tire_d / 2,
                  'rfw_c': veh.lcgf + -1 * veh.tire_d / 2,
                  'rfw_d': veh.lcgf + -1 * veh.tire_d / 2,
                  'rrw':   -1 * veh.lcgr,                          # Right rear wheel
                  'rrw_a': -1 * veh.lcgr + veh.tire_d / 2,
                  'rrw_b': -1 * veh.lcgr + veh.tire_d / 2,
                  'rrw_c': -1 * veh.lcgr - veh.tire_d / 2,
                  'rrw_d': -1 * veh.lcgr - veh.tire_d / 2,
                  'lrw':   -1 * veh.lcgr,                          # Left rear wheel
                  'lrw_a': -1 * veh.lcgr + veh.tire_d / 2,
                  'lrw_b': -1 * veh.lcgr + veh.tire_d / 2,
                  'lrw_c': -1 * veh.lcgr - veh.tire_d / 2,
                  'lrw_d': -1 * veh.lcgr - veh.tire_d / 2,
                  'cg': 0,                                         # CG
                  'xaxis': veh.lcgf + veh.f_hang + 1.5,           # line for x-axis
                  'yaxis': 0,                                      # line for y-axis
                  'vel_v': veh.vx_initial}

        veh.py = {'b_lfc': -1 * veh.width / 2,     # body outline
                  'b_rfc': veh.width / 2,
                  'b_rrc': veh.width / 2,
                  'b_lrc': -1 * veh.width / 2,
                  'lfw':   -1 * (veh.width / 2 - veh.tire_w / 2),                         # left front wheel
                  'lfw_a': -1 * (veh.width / 2),
                  'lfw_b': -1 * (veh.width / 2 - veh.tire_w),
                  'lfw_c': -1 * (veh.width / 2 - veh.tire_w),
                  'lfw_d': -1 * (veh.width / 2),
                  'rfw':   (veh.width / 2 - veh.tire_w / 2),                         # Right front wheel
                  'rfw_a': (veh.width / 2 - veh.tire_w),
                  'rfw_b': (veh.width / 2),
                  'rfw_c': (veh.width / 2),
                  'rfw_d': (veh.width / 2 - veh.tire_w),
                  'rrw':   (veh.width / 2 - veh.tire_w / 2),                          # Right rear wheel
                  'rrw_a': veh.width / 2 - veh.tire_w,
                  'rrw_b': veh.width / 2,
                  'rrw_c': veh.width / 2,
                  'rrw_d': veh.width / 2 - veh.tire_w,
                  'lrw': -1 * (veh.width / 2 - veh.tire_w / 2),                          # Left rear wheel
                  'lrw_a': -1 * veh.width / 2,
                  'lrw_b': -1 * veh.width / 2 + veh.tire_w,
                  'lrw_c': -1 * veh.width / 2 + veh.tire_w,
                  'lrw_d': -1 * veh.width / 2,
                  'cg': 0,                                         # CG
                  'xaxis': 0,   # line for x-axis
                  'yaxis': veh.width / 2 + 1.5,                                      # line for y-axis
                  'vel_v': veh.vy_initial}

        veh.Px = veh.px.copy()
        veh.Py = veh.py.copy()

        # rotate vehicle coordinates using initial heading angle (head_angle)
        for key in veh.Px:
            veh.Px[key] = veh.init_x_pos + veh.px[key]*np.cos(veh.head_angle * np.pi / 180) - veh.py[key]*np.sin(veh.head_angle * np.pi / 180)
            veh.Py[key] = veh.init_y_pos + veh.px[key]*np.sin(veh.head_angle * np.pi / 180) + veh.py[key]*np.cos(veh.head_angle * np.pi / 180)

        if veh.striking:
            # impact point (veh1 - striking vehicle)
            veh.impact_points_global = []
            veh.impact_norm_global = []
            veh.impact_tang_global = []

            for impPoints in veh.impact_points:
                """ create list of impact points in the global reference frame """
                pimpact_X = veh.init_x_pos + impPoints[0] * np.cos(veh.head_angle * np.pi / 180) - impPoints[1] * np.sin(veh.head_angle * np.pi / 180)
                pimpact_Y = veh.init_y_pos + impPoints[0] * np.sin(veh.head_angle * np.pi / 180) + impPoints[1] * np.cos(veh.head_angle * np.pi / 180)

                veh.impact_points_global.append((pimpact_X, pimpact_Y))

                norm_length = veh.width
                tang_length = veh.width / 2

                veh.impact_norm_global.append((pimpact_X + norm_length * np.cos(impPoints[2] * np.pi / 180),
                                              pimpact_Y + norm_length * np.sin(impPoints[2] * np.pi / 180)))

                veh.impact_tang_global.append((pimpact_X + tang_length * np.cos(impPoints[2] * np.pi / 180 + np.pi / 2),
                                              pimpact_Y + tang_length * np.sin(impPoints[2] * np.pi / 180 + np.pi / 2)))


    return vehicle_list



"""
plots used within vehicle class
"""
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import plotly.io as pio

pio.renderers.default = "browser"  # <- determines how plots are displayed using Plotly

# TODO: create input for figure size - loads from "defaults" folder?
figure_size = (1200, 700)
font_size = 24
tick_font_size = 22

"""
generating impact point - striking vehicle
"""


def plot_impact_points(veh, iplane=True):
    # x,y coordinates of vehicle outline:
    # left front corner
    veh._b_lfc_x = veh.lcgf + veh.f_hang
    veh._b_lfc_y = -1 * veh.width / 2
    # right front corner
    veh._b_rfc_x = veh.lcgf + veh.f_hang
    veh._b_rfc_y = veh.width / 2
    # right rear corner
    veh._b_rrc_x = -1 * veh.lcgr - veh.r_hang
    veh._b_rrc_y = veh.width / 2
    # left rear corner
    veh._b_lrc_x = -1 * veh.lcgr - veh.r_hang
    veh._b_lrc_y = -1 * veh.width / 2

    bdy_x = (veh._b_lfc_x, veh._b_rfc_x, veh._b_rrc_x, veh._b_lrc_x, veh._b_lfc_x)
    bdy_y = (veh._b_lfc_y, veh._b_rfc_y, veh._b_rrc_y, veh._b_lrc_y, veh._b_lfc_y)

    x_axis_length = veh._b_lfc_x * 1.5 - veh._b_lrc_x * 1.5

    # generate plot to show vehicle outline and default points for impact
    fig = go.Figure()

    # body outline
    fig.add_trace(go.Scatter(x=bdy_x, y=bdy_y,
                             mode='lines+markers',
                             line=dict(color='rgb(67,67,67)', width=2),
                             marker=dict(color='rgb(255, 0, 0)', size=8)))
    # cg
    fig.add_trace(go.Scatter(x=[0, 0], y=[0, 0],
                             mode='markers',
                             marker=dict(color='rgb(51, 204, 51)', size=20)))

    fig.add_annotation(x=1,
                       y=-1,
                       showarrow=False,
                       text="CG")
    # vehicle x-axis
    fig.add_annotation(x=5,
                       y=0,
                       axref='x',
                       ayref='y',
                       text="",
                       showarrow=True,
                       ax=0,
                       ay=0,
                       arrowsize=2,
                       arrowhead=1,
                       arrowwidth=1.5,
                       arrowcolor='rgb(0, 0, 0)')
    # vehicle y-axis
    fig.add_annotation(x=0,
                       y=5,
                       axref='x',
                       ayref='y',
                       text="",
                       showarrow=True,
                       ax=0,
                       ay=0,
                       arrowsize=2,
                       arrowhead=1,
                       arrowwidth=1.5,
                       arrowcolor='rgb(0, 0, 255)')


    fig.add_annotation(x=veh._b_lfc_x * 1.1,
                       y=veh._b_lfc_y,
                       showarrow=False,
                       text="1")
    fig.add_annotation(x=veh._b_rfc_x * 1.1,
                       y=veh._b_rfc_y,
                       showarrow=False,
                       text="2")
    fig.add_annotation(x=veh._b_rrc_x * 1.1,
                       y=veh._b_rrc_y,
                       showarrow=False,
                       text="3")
    fig.add_annotation(x=veh._b_lrc_x * 1.1,
                       y=veh._b_lrc_y,
                       showarrow=False,
                       text="4")

    if iplane:
        for i in range(len(veh.impact_points)):
            # normal impact plane
            fig.add_annotation(x=veh.impact_norm[i][0],
                               y=veh.impact_norm[i][1],
                               axref='x',
                               ayref='y',
                               text="",
                               showarrow=True,
                               ax=veh.impact_points[i][0],
                               ay=veh.impact_points[i][1],
                               arrowsize=2,
                               arrowhead=2,
                               arrowwidth=1.5,
                               arrowcolor='rgb(153, 0, 51)')
            # tangential impact plane
            fig.add_annotation(x=veh.impact_tang[i][0],
                               y=veh.impact_tang[i][1],
                               axref='x',
                               ayref='y',
                               text="",
                               showarrow=True,
                               ax=veh.impact_points[i][0],
                               ay=veh.impact_points[i][1],
                               arrowsize=1,
                               arrowhead=1,
                               arrowwidth=1,
                               arrowcolor='rgb(153, 0, 51)')

            fig.add_trace(go.Scatter(x=[veh.impact_points[i][0], veh.impact_points[i][0]],
                                     y=[veh.impact_points[i][1], veh.impact_points[i][1]],
                                     mode='markers',
                                     name=f'POI: {i}',
                                     marker=dict(color='rgb(153, 0, 51)', size=7),
                                     ))

    fig.update_layout(
        showlegend=False,
        autosize=False,
        width=figure_size[0],
        height=figure_size[1],
        title='Selecting Impact Point for Striking Vehicle',
        template='plotly_white',
        xaxis=dict(showgrid=False, title='x-axis - Forward (ft)'),
        yaxis=dict(showgrid=False, title='y-axis - Rightward (ft)'),
        font=dict(family='Arial', size=16, color='black'))

    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False,
                     range=[veh._b_lrc_x * 1.5, veh._b_lfc_x * 1.5])
    fig.update_yaxes(autorange='reversed', showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False,
                     range=[-0.5 * x_axis_length * figure_size[1] / figure_size[0],
                            0.5 * x_axis_length * figure_size[1] / figure_size[0]])
    fig.show()

    """
    generating impacting edge - struck vehicle
    """


def plot_impact_edge(veh):
    # x,y coordinates of vehicle outline:
    # left front corner
    veh._b_lfc_x = veh.lcgf + veh.f_hang
    veh._b_lfc_y = -1 * veh.width / 2
    # right front corner
    veh._b_rfc_x = veh.lcgf + veh.f_hang
    veh._b_rfc_y = veh.width / 2
    # right rear corner
    veh._b_rrc_x = -1 * veh.lcgr - veh.r_hang
    veh._b_rrc_y = veh.width / 2
    # left rear corner
    veh._b_lrc_x = -1 * veh.lcgr - veh.r_hang
    veh._b_lrc_y = -1 * veh.width / 2

    bdy_x = (veh._b_lfc_x, veh._b_rfc_x, veh._b_rrc_x, veh._b_lrc_x, veh._b_lfc_x)
    bdy_y = (veh._b_lfc_y, veh._b_rfc_y, veh._b_rrc_y, veh._b_lrc_y, veh._b_lfc_y)

    x_axis_length = veh._b_lfc_x * 1.5 - veh._b_lrc_x * 1.5

    # generate plot to show vehicle outline and default points for impact
    fig = go.Figure()

    # body outline
    fig.add_trace(go.Scatter(x=bdy_x[:2], y=bdy_y[:2],
                             mode='lines+text',
                             line=dict(color='rgb(0, 0, 0)', width=2)))
    fig.add_trace(go.Scatter(x=bdy_x[1:3], y=bdy_y[1:3],
                             mode='lines+text',
                             line=dict(color='rgb(51, 102, 204)', width=2)))
    fig.add_trace(go.Scatter(x=bdy_x[2:4], y=bdy_y[2:4],
                             mode='lines+text',
                             line=dict(color='rgb(0, 153, 51)', width=2)))
    fig.add_trace(go.Scatter(x=bdy_x[3:5], y=bdy_y[3:5],
                             mode='lines+text',
                             line=dict(color='rgb(255, 102, 0)', width=2)))

    # cg
    fig.add_trace(go.Scatter(x=[0, 0], y=[0, 0],
                             mode='markers',
                             marker=dict(color='rgb(51, 204, 51)', size=20)))

    # edge labels
    x_coordinates = [veh._b_lfc_x * 1.1, -2, veh._b_rrc_x * 1.1, -2]
    y_coordinates = [0, 1.2 * veh.width / 2, 0, -1.2 * veh.width / 2]
    for i in range(0, 4):
        if i in veh.edgeimpact:
            text = f'Edge {i} - POI {veh.edgeimpact.index(i)}'
        else:
            text = f'Edge {i}'
            
        fig.add_annotation(x=x_coordinates[i],
                           y=y_coordinates[i],
                           showarrow=False,
                           text=text)


    fig.add_annotation(x=1,
                       y=-1,
                       showarrow=False,
                       text="CG")
    fig.add_annotation(x=5,
                       y=0,
                       axref='x',
                       ayref='y',
                       text="",
                       showarrow=True,
                       ax=0,
                       ay=0,
                       arrowsize=2,
                       arrowhead=1,
                       arrowwidth=1.5,
                       arrowcolor='rgb(0, 0, 0)')

    fig.add_annotation(x=0,
                       y=5,
                       axref='x',
                       ayref='y',
                       text="",
                       showarrow=True,
                       ax=0,
                       ay=0,
                       arrowsize=2,
                       arrowhead=1,
                       arrowwidth=1.5,
                       arrowcolor='rgb(0, 0, 255)')

    fig.update_layout(
        showlegend=False,
        autosize=False,
        width=figure_size[0],
        height=figure_size[1],
        title='Selecting Impact Edge for Struck Vehicle',
        template='plotly_white',
        xaxis=dict(showgrid=False, title='x-axis - Forward (ft)'),
        yaxis=dict(showgrid=False, title='y-axis - Rightward (ft)'),
        font=dict(family='Arial', size=16, color='black'))

    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False,
                     range=[veh._b_lrc_x * 1.5, veh._b_lfc_x * 1.5])
    fig.update_yaxes(autorange='reversed', showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False,
                     range=[-0.5 * x_axis_length * figure_size[1] / figure_size[0],
                            0.5 * x_axis_length * figure_size[1] / figure_size[0]])
    fig.show()


"""
plot driver inputs
"""


def plot_driver_inputs(self):
    fig = make_subplots(specs=[[{"secondary_y": True}]])

    fig.add_trace(go.Scatter(x=self.driver_input.t, y=self.driver_input.throttle * 100,
                             mode='lines',
                             name='throttle',
                             line=dict(color='rgb(0, 255, 0)', width=2)),
                  secondary_y=False)
    fig.add_trace(go.Scatter(x=self.driver_input.t, y=self.driver_input.brake * 100,
                             mode='lines',
                             name='brake',
                             line=dict(color='rgb(255, 0, 0)', width=2)),
                  secondary_y=False)
    fig.add_trace(go.Scatter(x=self.driver_input.t, y=self.driver_input.steer,
                             mode='lines',
                             name='steer',
                             line=dict(color='rgb(0, 0, 0)', width=2)),
                  secondary_y=True)

    fig.update_layout(
        legend=dict(orientation="h", yanchor='top', y=1.05, xanchor='left', x=0.45),
        autosize=False,
        width=figure_size[0],
        height=figure_size[1],
        title=f'Driver Inputs for {self.name}',
        template='plotly_white',
        xaxis=dict(showgrid=False, title='Time (s)'),
        yaxis=dict(showgrid=False),
        font=dict(family='Arial', size=font_size, color='black'))

    fig.update_yaxes(showgrid=False, title_text='Steer Angle (deg)', secondary_y=True)
    fig.update_yaxes(showgrid=False, title_text='Brake | Throttle (%)', secondary_y=False)

    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, tickfont=dict(size=tick_font_size))
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, tickfont=dict(size=tick_font_size))
    fig.show()



from .model_calcs.tire_model import tire_forces
import pandas as pd
import numpy as np
from scipy import integrate
import math

"""
comprehensive vehicle model that can be used to run single vehicle motion as well as
impact related motion
"""

# TODO: ignore driver inputs after impact
# TODO: disable tire after impact

def multi_vehicle_model(veh, i, sim_defaults, impact_type, ignore_driver=False, kmutual=None, vehicle_mu=None):
    """
    Calculate vehicle dynamics from driver inputs and environmental inputs
    vehicle_list is a list of vehicle class instances [veh1, veh2] - two currently
    model will use the max time from vehicle 1 for input, all vehicles
    need the same total driver input time
    model_type indicates the type of impact to be simulated (sideswipe (ss), or impulse-momentum (impc))
    ignore_driver (False) will use driver_inputs after impact.  True will ignore all driver inputs and keep
    last input for remainder of simulation
    kmutual must be defined by sideswipe simulations
    """
    #print(f'i in vehicle model {i}')

    dt_motion = sim_defaults['dt_motion']
    veh.model.t[i] = round(i * dt_motion, 4)  # assigning time
    #print(round(i * dt_motion, 4))
    #print(veh.model.t[i])
    # get tire forces for t = 0
    veh = tire_forces(veh, i, sim_defaults)

    # setting vehicle forces to zero if no impact
    # impact may occur as a result of vehicle 2 motion in which case, the forces for t=i will be
    # updated for each vehicle
    # TODO: update for sideswipe / force model
    veh.model.Fx[i] = 0
    veh.model.Fy[i] = 0
    veh.model.Mz[i] = 0
    """
    if impact_type != 'SS':
        veh.model.Fx[i] = 0
        veh.model.Fy[i] = 0
        veh.model.Mz[i] = 0
    elif (impact_type == 'SS') & (impact == False):
        veh.model.Fx[i] = 0
        veh.model.Fy[i] = 0
        veh.model.Mz[i] = 0
    """

    # local vehicle acceleration
    veh.model.au[i] = 32.2 / veh.weight * np.sum([veh.model.lf_fx[i],
                                                  veh.model.rf_fx[i],
                                                  veh.model.rr_fx[i],
                                                  veh.model.lr_fx[i],
                                                  veh.model.Fx[i]])

    veh.model.av[i] = 32.2 / veh.weight * np.sum([veh.model.lf_fy[i],
                                                  veh.model.rf_fy[i],
                                                  veh.model.rr_fy[i],
                                                  veh.model.lr_fy[i],
                                                  veh.model.Fy[i]])

    # rotation acceleration - alpha-z
    veh.model.alphaz[i] = (1 / veh.izz) * np.sum([veh.model.lf_fx[i] * veh.track / 2,
                                                  veh.model.lf_fy[i] * veh.lcgf,
                                                  -1 * veh.model.rf_fx[i] * veh.track / 2,
                                                  veh.model.rf_fy[i] * veh.lcgf,
                                                  -1 * veh.model.rr_fx[i] * veh.track / 2,
                                                  -1 * veh.model.rr_fy[i] * veh.lcgr,
                                                  veh.model.lr_fx[i] * veh.track / 2,
                                                  -1 * veh.model.lr_fy[i] * veh.lcgr,
                                                 veh.model.Mz[i]])

    if i == 0:
        # vehicle acceleration in inertial frame
        veh.model.ax[i] = veh.model.au[i] + veh.model.oz_rad[i] * veh.model.vy[i]
        veh.model.ay[i] = veh.model.av[i] - veh.model.oz_rad[i] * veh.model.vx[i]
        veh.model.ar[i] = np.sqrt(veh.model.ax[i] ** 2 + veh.model.ay[i] ** 2)

        # inertial frame coorindates - capital letters
        veh.model.Ax[i] = veh.model.au[i] * np.cos(veh.model.theta_rad[i]) - veh.model.av[i] * np.sin(veh.model.theta_rad[i])
        veh.model.Ay[i] = veh.model.au[i] * np.sin(veh.model.theta_rad[i]) + veh.model.av[i] * np.cos(veh.model.theta_rad[i])

    if i > 0:
        # vehicle acceleration in inertial frame
        veh.model.ax[i] = veh.model.au[i] + veh.model.oz_rad[i - 1] * veh.model.vy[i - 1]
        veh.model.ay[i] = veh.model.av[i] - veh.model.oz_rad[i - 1] * veh.model.vx[i - 1]
        veh.model.ar[i] = np.sqrt(veh.model.ax[i] ** 2 + veh.model.ay[i] ** 2)

        # vehicle velocities
        veh.model.vx[i] = veh.model.vx[i - 1] + dt_motion * np.mean([veh.model.ax[i - 1], veh.model.ax[i]])
        veh.model.vy[i] = veh.model.vy[i - 1] + dt_motion * np.mean([veh.model.ay[i - 1], veh.model.ay[i]])

        # omega
        veh.model.oz_rad[i] = veh.model.oz_rad[i - 1] + dt_motion * np.mean([veh.model.alphaz[i - 1], veh.model.alphaz[i]])

        # heading angle
        veh.model.theta_rad[i] = veh.model.theta_rad[i - 1] + dt_motion * np.mean([veh.model.oz_rad[i], veh.model.oz_rad[i - 1]])

        # inertial frame coorindates - capital letters
        veh.model.Ax[i] = veh.model.au[i] * np.cos(veh.model.theta_rad[i]) - veh.model.av[i] * np.sin(veh.model.theta_rad[i])
        veh.model.Ay[i] = veh.model.au[i] * np.sin(veh.model.theta_rad[i]) + veh.model.av[i] * np.cos(veh.model.theta_rad[i])

        veh.model.Vx[i] = veh.model.Vx[i - 1] + dt_motion * np.mean([veh.model.Ax[i - 1], veh.model.Ax[i]])
        veh.model.Vy[i] = veh.model.Vy[i - 1] + dt_motion * np.mean([veh.model.Ay[i - 1], veh.model.Ay[i]])

        # velocity vector in inertial frame
        # TODO: use numpy arctan2?
        veh.model.beta_rad[i] = math.atan2(veh.model.Vy[i], veh.model.Vx[i])  # move to seperate calc

    # vehicle position
    veh.model['Dx'] = veh.init_x_pos + integrate.cumtrapz(list(veh.model.Vx), list(veh.model.t), initial=0)
    veh.model['Dy'] = veh.init_y_pos + integrate.cumtrapz(list(veh.model.Vy), list(veh.model.t), initial=0)

    return veh



"""
class for static impulse momentum model
"""
from pycrash.impc import impc_calcs
import numpy as np
from copy import deepcopy


class IMPC():
    def __init__(self, name, veh1, veh2, sim_inputs):
        self.name = name
        self.type = 'impc_carpenter'  # class type for saving files
        self.veh1 = deepcopy(veh1)
        self.veh2 = deepcopy(veh2)

        impactp_gx = self.veh1.init_x_pos + self.veh1.pimpact_x * np.cos(self.veh1.head_angle * np.pi / 180) - self.veh1.pimpact_y * np.sin(self.veh1.head_angle * np.pi / 180)
        impactp_gy = self.veh1.init_y_pos + self.veh1.pimpact_x * np.sin(self.veh1.head_angle * np.pi / 180) + self.veh1.pimpact_y * np.cos(self.veh1.head_angle * np.pi / 180)
        self.poi_veh2x = ((impactp_gx-self.veh2.init_x_pos) * np.cos(self.veh2.head_angle * np.pi / 180) + (impactp_gy-self.veh2.init_y_pos) * np.sin(self.veh2.head_angle * np.pi / 180))
        self.poi_veh2y = (-1 * (impactp_gx-self.veh2.init_x_pos) * np.sin(self.veh2.head_angle * np.pi / 180) + (impactp_gy-self.veh2.init_y_pos) * np.cos(self.veh2.head_angle * np.pi / 180))

        # convert input speeds to fps
        self.veh1.vx_initial = self.veh1.vx_initial * 1.46667
        self.veh1.vy_initial = self.veh1.vy_initial * 1.46667
        self.veh2.vx_initial = self.veh2.vx_initial * 1.46667
        self.veh2.vy_initial = self.veh2.vy_initial * 1.46667

        self.v1_result, self.v2_result = impc_calcs(self.veh1, self.veh2, self.poi_veh2x, self.poi_veh2y, sim_inputs)

    def results(self):
        print('Vehicle 1:')
        print(self.v1_result)
        print(f"PDOF Veh1: {np.arctan(self.v1_result['dvy'] / self.v1_result['dvx']) * 180 / np.pi}")
        print('')
        print('Vehicle 2:')
        print(self.v2_result)
        print(f"PDOF Veh2: {np.arctan(self.v2_result['dvy'] / self.v2_result['dvx']) * 180 / np.pi}")



from .project import Project, project_info, load_project
from .impact_main import Impact
from .vehicle import Vehicle
from .kinematics import SingleMotion
from .kinematicstwo import KinematicsTwo
from .sdof_model import SDOF_Model
from .definitions import definitions

__all__ = ["Impact", "Project", "Vehicle", "SingleMotion", "KinematicsTwo", "SDOF_Model", "definitions"]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pycrash",
    version="0.0.17",
    author="Joe Cormier",
    author_email="joemcormier@outlook.com",
    description="software tool for simulating vehicle motion and impacts based on \
    fundamental physics and accident reconstruction techniques",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/joe-cormier/pycrash",
    packages=setuptools.find_packages(exclude=("__pycache__", )),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
